from django.core.management import call_command

from .check__base import CheckBase


class MigrateCheck(CheckBase):
    """Check migrations."""

    message = "MIGRATIONS ARE APPLIED"

    def check(self) -> bool:
        """Check method implementation."""
        try:
            call_command("migrate", "--check", no_input=True)
            self._print_check(self.message)
            return True
        except SystemExit:
            self._print_check(self.message, False)
        except Exception as exc:
            self._print_check(self.message, exception=exc)
        return False



from typing import Any, Dict, Tuple

from django.contrib.auth.models import User
from django.core.management import BaseCommand, call_command


class Command(BaseCommand):
    """Init Django command."""

    help = "Init Django (create caches, migrate, create admin user, collect static)."  # noqa: A003

    def handle(self, *args: Tuple[Any, ...], **options: Dict[str, Any]) -> None:  # noqa: ARG002
        """Init Django implementation."""
        call_command("createcachetable")
        call_command("migrate", no_input=True, interactive=False)

        if not User.objects.filter(is_superuser=True).exists():
            User.objects.create_superuser(
                "admin",
                email=None,
                password="admin",  # pragma: allowlist secret # noqa: S106
            )

        call_command("collectstatic", no_input=True, interactive=False)



import random
from abc import ABCMeta, abstractmethod
from typing import Optional

from django.conf import settings
from django.core.management.base import OutputWrapper


class CheckBase(metaclass=ABCMeta):
    """Waiter check base class."""

    message_template = ""
    message = ""
    extra_info: Optional[str] = None

    def __init__(self, handler_stdout: OutputWrapper) -> None:
        self.stdout: OutputWrapper = handler_stdout

    @abstractmethod
    def check(self) -> bool:
        """Check method implementation."""
        raise NotImplementedError

    def _print_check(
        self,
        label: str,
        success: Optional[bool] = True,
        *,
        width: int = 96,
        exta_info: Optional[str] = None,
        exception: Optional[Exception] = None,
    ) -> None:
        """Log a message with a label and a status indicator.

        Args:
        ----
            label: A message label.
            success: A status indicator. Defaults to True. If None, the status indicator is SKIPPED.
            width: A width of the message. Defaults to 80.
            exta_info: Extra info string. Defaults to None.
            exception: An exception instance. Defaults to None.

        Returns:
        -------
            None
        """
        status_indicator = "SKIPPED" if success is None else ("OK" if success else "FAILED")
        if isinstance(exception, Exception):
            status_indicator = "FAILED"

        spacers = width - len(label) - len(status_indicator)
        msg = f"{label}{'.' * spacers}{status_indicator}"
        self.stdout.write(msg)

        if exta_info is not None:
            self.stdout.write(exta_info + "\n")

        if isinstance(exception, Exception):
            self.stdout.write(f"{exception.__class__.__name__}: {exception}" + "\n" * 3)

    @staticmethod
    def _get_random_hexstr(length: int = 8) -> str:
        """Generate a random hash.

        Args:
        ----
            length: A length of the hash. Defaults to 8.

        Returns:
        -------
            A random hash.
        """
        return "".join(random.choices("0123456789abcdef", k=length))


class DbInfoMixin:
    """Database info mixin."""

    @staticmethod
    def _get_database_extra_info(alias: str) -> Optional[str]:
        """Get extra info about database by alias.

        Args:
        ----
            alias: A database alias.

        Returns:
        -------
            Extra info about database.
        """
        return (
            (
                f"{settings.DATABASES[alias]['ENGINE']}//"
                f"{settings.DATABASES[alias]['USER']}@"
                f"{settings.DATABASES[alias]['HOST']}:"
                f"{settings.DATABASES[alias]['PORT']}/"
                f"{settings.DATABASES[alias]['NAME']}"
            )
            if alias in settings.DATABASES
            else None
        )



from django.conf import settings
from django.db import connections
from django.db.backends.utils import CursorWrapper  # noqa

from .check__base import CheckBase, DbInfoMixin


class DbWriteCheck(CheckBase, DbInfoMixin):
    """Check database write availability."""

    message_template = "DB CAN WRITE [{}]"

    def check(self) -> bool:
        """Check method implementation."""
        try:
            test_table_name = f"__test_{self._get_random_hexstr()}__"
            for alias in settings.DATABASES:
                self.message = self.message_template.format(alias)
                self.extra_info = self._get_database_extra_info(alias)
                with connections[alias].cursor() as cursor:  # type: CursorWrapper
                    cursor.execute(f"CREATE TABLE {test_table_name} (id serial PRIMARY KEY, num integer);")
                    cursor.execute(f"INSERT INTO {test_table_name} (num) VALUES (1);")
                    cursor.execute(f"UPDATE {test_table_name} SET num = 2 WHERE id = 1;")
                    cursor.execute(f"DELETE FROM {test_table_name} WHERE id = 1;")
                    cursor.execute(f"DROP TABLE {test_table_name};")
                    self._print_check(self.message)
            return True
        except Exception as exc:
            self._print_check(self.message, exception=exc, exta_info=self.extra_info)
        return False



import importlib
import os
import sys
from typing import Any, Optional

from django.conf import settings

from .check__base import CheckBase


class CeleryBrokerAvailableCheck(CheckBase):
    """Check celery broker availability."""

    message = "CELERY IS AVAILABLE"

    def check(self) -> bool:
        """Check method implementation."""
        celery_url = getattr(settings, "CELERY_BROKER_URL", getattr(settings, "BROKER_URL", None))
        if celery_url is None:
            self._print_check("CELERY IS NOT CONFIGURED", None)
            return True

        celery_app = self._find_celery_module()
        if celery_app is None:
            self._print_check("CELERY APP IS NOT FOUND", False)
            return False

        try:
            celery_inspect = importlib.import_module("celery.app.control").Inspect
            celery_inspect(app=celery_app, timeout=0.25).ping()  # throws exception if broker isnt available
            self._print_check(self.message)
            return True
        except Exception as exc:
            self._print_check(self.message, exception=exc, exta_info=f"Broker URL: {celery_url}")
        return False

    @staticmethod
    def _find_celery_module() -> Optional[Any]:
        """Find celery module."""
        work_dir = os.getcwd()
        for root, dirs, files in os.walk(work_dir):
            if os.path.relpath(root, work_dir).count(os.sep) < 3 and "celery.py" in files:
                sys.path.append(root)
                module_name = f"{os.path.basename(root)}.celery"
                try:
                    module = importlib.import_module(module_name)
                    celery_app = getattr(module, "celery_app", None)
                    if celery_app is not None:
                        return celery_app
                    del sys.modules[module_name]
                except ModuleNotFoundError:
                    continue
        return None
